#include <string.h>

typedef struct
  {
   char m_VLTY[1028]; // ("HFS")
   char m_VLNM[1028]; // ("b4028aef-2")
   char m_DSNM[1028]; //  ("14:Track1.trk")
  } OpenTLFile_t;

// from doc
//TKNM  Track Name                  String
//STTM  Start Time                  Sample Number
//EDLS  Edit List                   { List of EVNT }
//EVNT  Event                       { List of Simple Properties }

//EDNM  Edit Name                   String
//EDPT  Edit Point                  Sample Number
//EDLN  Edit Length                 Integer (>= 0)
//RULN  RampUp Length               Integer (>= 0)
//RUOF  RampUp Offset               Integer (>= 0)
//RUSH  RampUp Shape                Enumerated Type ( 0 = Linear )
//RDLN  RampDown Length             Integer (>= 0)
//RDOF  RampDown Offset             Integer (>= 0)
//RDSH  RampDown Shape              Enumerated Type ( 0 = Linear )
//INGN  Initial Gain           Floating Point(0.0 - 1.0) Linear Gain (Amplitude)
//FNGN  Final Gain             Floating Point(0.0 - 1.0) Linear Gain (Amplitude)
//EVMU  Event Muted                 0 or 1
//DSOF  DataSet Offset              Sample Number
//SNTY  Sound Data Type             String ("SDII", "WAV")
//EVDS  Event FILE reference        FILE complex value

typedef struct
  {
  char m_strEDNM_EditName[128];
  unsigned long m_ulEDPT_EditPoint;
  unsigned long m_ulEDLN_EditLength;
  unsigned long m_ulRULN_RampUpLength;
  unsigned long m_ulRUOF_RampUpOffset;
  unsigned int m_unRUSH_RampUpShape;
  unsigned long m_ulRDLN_RampDownLength;
  unsigned long m_ulRDOF_RampDownOffset;
  unsigned int m_unRDSH_RampDownShape;
  float m_fINGN_InitialGain;
  float m_fFNGN_FinalGain;
  unsigned int m_bEVMU_EventMuted;
  unsigned long m_ulDSOF_DataSetOffset;
  char m_strSNTY_SoundDataType[10];
  OpenTLFile_t m_fileEVDS_EventFILEreference;
  } OpenTLEvent_t;

// from doc
// - optional Gain list (GNLS) may follow required EDLS - 
//
//GNLS  Gain List  (Volume auto.)   { List of GNEV } 
//GNEV  Gain Event (volume level)   { List of Simple Properties }
//GNPT  Gain Point                  Sample Number
//GNLV  Gain Level        Floating Point (0.0 - 1.0) (Slider Value or Fader Level)
//

typedef struct
  {
  unsigned long m_ulGNPT_GainPoint;
  float m_fGNLV_GainLevel;
  } OpenTLGainEvent_t;

// from doc
// - optional Mute list (MULS) may also follow required EDLS - 
//MULS  Mute List  (Mute auto)      { List of MUEV } 
//MUEV  Mute Event (Mute On/Off)    { List of Simple Properties }
//MUPT  Mute Point                  Sample Number
//MULV  Integer                     (0 = OFF, 1 = ON)
//

typedef struct
  {
  unsigned long m_ulMUPT_MutePoint;
  unsigned int m_bMULV_Mute;
  } OpenTLMuteEvent_t;



class MyReader
{

public:

  MyReader()
    {}

  virtual ~MyReader()
    {}

// MyReader members to collect data from parser

// from doc
//PJNM  Project Name               String
//SMRT  Sample Rate                Integer (see below)
//FRRT  Frame Rate                 Integer (see below)
//SASI  Sample Size in Bytes       Integer
//SNTY  Sound Type For Record      String ("SDII", "WAV")
//PUSQ  Punch Sequence             Integer (last punch sequence used by proj)
//TKSQ  Track Sequence             Integer (last track sequence number used)
//TKLS  Track List                 { List of TRAK }
//TRAK  Track File                 { List of Simple Properties }
//TKNM  Track Name                 String
//TKIN  Track Index                Integer (assigned track output)
//                                          A value higher than is supported
//                                          by a system is treated as virtual.
//TKDS  Track FILE Reference       FILE complex value

  char m_strPJNM_ProjectName[128];
  unsigned int m_unSMRT_SampleRate;
  unsigned int m_unFRRT_FrameRate;
  unsigned int m_unSASI_SampleSizeinBytes;
  char m_strSNTY_SoundTypeForRecord[10];
  unsigned int m_unPUSQ_PunchSequence;
  unsigned int m_unTKSQ_TrackSequence;
  char m_strTKNM_TrackName[128];
  unsigned int m_unTKIN_TrackIndex;
  unsigned int m_unTKOI_TrackOrigIndex;

  OpenTLFile_t m_fileTKDS_TrackFILEReference;

  void MySetPJNM_ProjectName(char* input)
    { 
    strcpy(m_strPJNM_ProjectName, input);
    printf("m_strPJNM_ProjectName: %s\n",m_strPJNM_ProjectName);
    }
  void MySetSMRT_SampleRate(unsigned int input)
    { 
    m_unSMRT_SampleRate = input;  
    printf("m_unSMRT_SampleRate: %d\n",m_unSMRT_SampleRate);
    }
  void MySetFRRT_FrameRate(unsigned int input)
    { 
    m_unFRRT_FrameRate = input;
    printf("m_unFRRT_FrameRate: %d\n",m_unFRRT_FrameRate);
    }
  void MySetSASI_SampleSizeinBytes(unsigned int input)
    { 
    m_unSASI_SampleSizeinBytes = input;
    printf("m_unSASI_SampleSizeinBytes: %d\n",m_unSASI_SampleSizeinBytes);
    }
  void MySetPUSQ_PunchSequence(unsigned int input)
    { 
    m_unPUSQ_PunchSequence = input;
    printf("m_unPUSQ_PunchSequence: %d\n",m_unPUSQ_PunchSequence);
    }
  void MySetTKSQ_TrackSequence(unsigned int input)
    { 
    m_unTKSQ_TrackSequence = input;
    printf("m_unTKSQ_TrackSequence: %d\n",m_unTKSQ_TrackSequence);
    }
  void MySetTKIN_TrackIndex(unsigned int input)
    { 
    m_unTKIN_TrackIndex = input;
    printf("m_unTKIN_TrackIndex: %d\n",m_unTKIN_TrackIndex);
    }
  void MySetTKOI_TrackOrigIndex(unsigned int input)
    { 
    m_unTKOI_TrackOrigIndex = input;
    printf("m_unTKOI_TrackOrigIndex: %d\n",m_unTKOI_TrackOrigIndex);
    }
  void MySetTKNM_TrackName(char* input)
    { 
    strcpy(m_strTKNM_TrackName, input);
    printf("m_strTKNM_TrackName: %s\n",m_strTKNM_TrackName);
    }
  void MySetTKDS_TrackFILEReference(OpenTLFile_t curfile)
    { 
    strcpy(m_fileTKDS_TrackFILEReference.m_VLTY, curfile.m_VLTY);
    strcpy(m_fileTKDS_TrackFILEReference.m_VLNM, curfile.m_VLNM);
    strcpy(m_fileTKDS_TrackFILEReference.m_DSNM, curfile.m_DSNM);
    printf("m_fileTKDS_TrackFILEReference:\n  m_VLTY: %s\n  m_VLNM: %s\n  m_DSNM: %s\n",
                    m_fileTKDS_TrackFILEReference.m_VLTY,
                    m_fileTKDS_TrackFILEReference.m_VLNM,
                    m_fileTKDS_TrackFILEReference.m_DSNM);
    }

  unsigned long m_ulSTTM_StartTime;

  void MySetSTTM_StartTime(unsigned long input)
    { 
    m_ulSTTM_StartTime = input; 
    printf("m_ulSTTM_StartTime: %d\n",m_ulSTTM_StartTime);
    }

  OpenTLEvent_t m_Event;

  void ClearEvent(OpenTLEvent_t* event)
    {
    strcpy(event->m_strEDNM_EditName, "");
    event->m_ulEDPT_EditPoint = 0; 
    event->m_ulEDLN_EditLength = 0; 
    event->m_ulRULN_RampUpLength = 0; 
    event->m_ulRUOF_RampUpOffset = 0; 
    event->m_unRUSH_RampUpShape = 0; 
    event->m_ulRDLN_RampDownLength = 0; 
    event->m_ulRDOF_RampDownOffset = 0; 
    event->m_unRDSH_RampDownShape = 0; 
    event->m_fINGN_InitialGain = 0.0f;
    event->m_fFNGN_FinalGain = 0.0f; 
    event->m_bEVMU_EventMuted = 0;
    event->m_ulDSOF_DataSetOffset = 0;
    strcpy(event->m_strSNTY_SoundDataType,"");
    strcpy(event->m_fileEVDS_EventFILEreference.m_VLTY,"");
    strcpy(event->m_fileEVDS_EventFILEreference.m_VLNM,"");
    strcpy(event->m_fileEVDS_EventFILEreference.m_DSNM,"");
    }

  void MySet_OpenTLEvent(OpenTLEvent_t event)
    {
    ClearEvent(&m_Event);
    strcpy(m_Event.m_strEDNM_EditName, event.m_strEDNM_EditName);
    m_Event.m_ulEDPT_EditPoint = event.m_ulEDPT_EditPoint;
    m_Event.m_ulEDLN_EditLength = event.m_ulEDLN_EditLength;
    m_Event.m_ulRULN_RampUpLength = event.m_ulRULN_RampUpLength;
    m_Event.m_ulRUOF_RampUpOffset = event.m_ulRUOF_RampUpOffset;
    m_Event.m_unRUSH_RampUpShape = event.m_unRUSH_RampUpShape;
    m_Event.m_ulRDLN_RampDownLength = event.m_ulRDLN_RampDownLength;
    m_Event.m_ulRDOF_RampDownOffset = event.m_ulRDOF_RampDownOffset;
    m_Event.m_unRDSH_RampDownShape = event.m_unRDSH_RampDownShape;
    m_Event.m_fINGN_InitialGain = event.m_fINGN_InitialGain;
    m_Event.m_fFNGN_FinalGain = event.m_fFNGN_FinalGain;
    m_Event.m_bEVMU_EventMuted = event.m_bEVMU_EventMuted;
    m_Event.m_ulDSOF_DataSetOffset = event.m_ulDSOF_DataSetOffset;
    strcpy(m_Event.m_strSNTY_SoundDataType, event.m_strSNTY_SoundDataType);
    strcpy(m_Event.m_fileEVDS_EventFILEreference.m_VLTY,event.m_fileEVDS_EventFILEreference.m_VLTY);
    strcpy(m_Event.m_fileEVDS_EventFILEreference.m_VLNM,event.m_fileEVDS_EventFILEreference.m_VLNM);
    strcpy(m_Event.m_fileEVDS_EventFILEreference.m_DSNM,event.m_fileEVDS_EventFILEreference.m_DSNM);

    printf("m_Event\n");
    printf("   m_strEDNM_EditName: %s\n",m_Event.m_strEDNM_EditName);
    printf("   m_ulEDPT_EditPoint: %d\n",m_Event.m_ulEDPT_EditPoint);
    printf("   m_ulEDLN_EditLength: %d\n",m_Event.m_ulEDLN_EditLength);
    printf("   m_ulRULN_RampUpLength: %d\n",m_Event.m_ulRULN_RampUpLength);
    printf("   m_ulRUOF_RampUpOffset: %d\n",m_Event.m_ulRUOF_RampUpOffset);
    printf("   m_unRUSH_RampUpShape: %d\n",m_Event.m_unRUSH_RampUpShape);
    printf("   m_ulRDLN_RampDownLength: %d\n",m_Event.m_ulRDLN_RampDownLength);
    printf("   m_ulRDOF_RampDownOffset: %d\n",m_Event.m_ulRDOF_RampDownOffset);
    printf("   m_unRDSH_RampDownShape: %d\n",m_Event.m_unRDSH_RampDownShape);
    printf("   m_fINGN_InitialGain: %f\n",m_Event.m_fINGN_InitialGain);
    printf("   m_fFNGN_FinalGain: %f\n",m_Event.m_fFNGN_FinalGain);
    printf("   m_bEVMU_EventMuted: %d\n",m_Event.m_bEVMU_EventMuted);
    printf("   m_ulDSOF_DataSetOffset: %d\n",m_Event.m_ulDSOF_DataSetOffset);
    printf("   m_strSNTY_SoundDataType: %s\n",m_Event.m_strSNTY_SoundDataType);
    printf("   m_fileEVDS_EventFILEreference......\n");
    printf("     m_VLTY: %s\n",m_Event.m_fileEVDS_EventFILEreference.m_VLTY);
    printf("     m_VLNM: %s\n",m_Event.m_fileEVDS_EventFILEreference.m_VLNM);
    printf("     m_DSNM: %s\n",m_Event.m_fileEVDS_EventFILEreference.m_DSNM);
    }

  OpenTLGainEvent_t m_GainEvent;

  void ClearGainEvent(OpenTLGainEvent_t* gainevent)
    {
    gainevent->m_ulGNPT_GainPoint = 0; 
    gainevent->m_fGNLV_GainLevel = 0.0f;
    }

  void MySet_OpenTLGainEvent(OpenTLGainEvent_t gainevent)
    {
    ClearGainEvent(&m_GainEvent);
    m_GainEvent.m_ulGNPT_GainPoint = gainevent.m_ulGNPT_GainPoint; 
    m_GainEvent.m_fGNLV_GainLevel = gainevent.m_fGNLV_GainLevel;
    printf("m_GainEvent\n");
    printf("   m_ulGNPT_GainPoint: %d\n",m_GainEvent.m_ulGNPT_GainPoint);
    printf("   m_fGNLV_GainLevel: %f\n",m_GainEvent.m_fGNLV_GainLevel);
    }

  OpenTLMuteEvent_t m_MuteEvent;

  void ClearMuteEvent(OpenTLMuteEvent_t* muteevent)
    {
    muteevent->m_ulMUPT_MutePoint = 0; 
    muteevent->m_bMULV_Mute = 0;
    }

  void MySet_OpenTLMuteEvent(OpenTLMuteEvent_t muteevent)
    {
    ClearMuteEvent(&m_MuteEvent);
    m_MuteEvent.m_ulMUPT_MutePoint = muteevent.m_ulMUPT_MutePoint; 
    m_MuteEvent.m_bMULV_Mute = muteevent.m_bMULV_Mute;
    printf("m_MuteEvent\n");
    printf("   m_ulMUPT_MutePoint: %d\n",m_MuteEvent.m_ulMUPT_MutePoint);
    printf("   m_bMULV_Mute: %d\n",m_MuteEvent.m_bMULV_Mute);
    }

};
