/* NAME
 *    utsalloc.cpp
 *
 * PROPRIETARY RIGHTS NOTICE
 *    All rights reserved. This material contains the valuable properties and
 *    trade secrets of TimeLine Vista, Inc., of Vista California, United States
 *    of America, embodying substantial creative efforts and confidential
 *    information, ideas and expressions, no part of which may be reproduced or
 *    transmitted in any form or by any means, electronic, mechanical, or
 *    otherwise, including photocopying and recording, or in connection with
 *    any information storage or retrieval system, without the permission in
 *    writing from TimeLine Vista, Inc.
 *
 * COPYRIGHT
 *    Copyright (C) 1994, an unpublished work, TimeLine Vista, Inc.
 *
 * DESCRIPTION
 *    Utility routines providing string memory allocation using a statically
 *    allocated fixed size buffer.
 *
*/


#include <stdlib.h>

#include "Utsalloc.hpp"

//-----------------------------------------------------------------------------
//                          class StringAllocator
//-----------------------------------------------------------------------------
StringAllocator::StringAllocator(ULONG poolSize)
{
   mBuff = (char *) malloc((size_t) poolSize);
   mIndex = 0;
}

StringAllocator::~StringAllocator()
{
   free(mBuff);
}

char* StringAllocator::stralloc(ULONG strSize)
{
   char* strptr = &mBuff[mIndex];

   mIndex += strSize;
   return strptr;
}


ULONG StringAllocator::strmark(void)
{
   return mIndex;
}

void StringAllocator::strRelease(ULONG lastMark)
{
   mIndex = lastMark;
}

void StringAllocator::strInit(void)
{
   mIndex = 0;
}
